# Copyright (c) 2003 Sylvain Thenault (thenault@nerim.net)
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
"""astng loader, avoid multible astng build of a same module
"""

__author__ = "Sylvain Thenault"
__revision__ = "$Id: manager.py,v 1.6 2003/09/12 11:54:49 syt Exp $"

import sys
from os.path import dirname
from logilab.common.modutils import load_module_from_name
from astng import ASTNGBuilder, ASTNGBuildingException, normalize_module_name
from astng_objects import ImportStmt

class ASTNGManager:
    """manage astng representation, avoiding to build a representation for a
    same module multiple times
    """
    
    def __init__(self):
        self._builder = ASTNGBuilder()
        self._modules = {}
        self._modules_by_name = {}

    def astng_from_class(self, classe, module=None):
        """ get astng for the given class """
        if module is None:
            try:
                module = classe.__module__
            except AttributeError:
                return #ClassDef()
        astng = self.astng_from_module_name(module)
        astng = astng.locals()[classe.__name__]    
        if isinstance(astng, ImportStmt):
            module = astng.base
            return self.astng_from_class(classe, module)
        return astng

    def astng_from_module(self, module, mod_name=None):
        """ given an imported module, return the astng object
        """
        try:
            return self._modules[module]
        except:
            if hasattr(module, '__file__'):
                # this is required to make works relative imports in
                # analyzed code
                sys.path.insert(0, dirname(module.__file__))
            try:
                astng = self._builder.build_from_module(module, mod_name)
                # update caches
                self._modules[module] = astng                
                self._modules_by_name[astng.__name__] = astng
                return astng
            finally:
                # clean path
                if hasattr(module, '__file__'):
                    sys.path.pop(0)

    def astng_from_module_name(self, mod_name):
        """ given a module name, return the astng object
        """
        norm_mod_name = normalize_module_name(mod_name)
        try:
            return self._modules_by_name[norm_mod_name]
        except KeyError:
            try:
                module = load_module_from_name(norm_mod_name)
            except Exception, ex:
                msg = 'Unable to load module %s (%s)' % (mod_name, ex)
                raise ASTNGBuildingException(msg)
        # return the astng representation
        return self.astng_from_module(module, norm_mod_name)


ASTNG_MANAGER = ASTNGManager()

if __name__ == '__main__':
    import sys
    obj_code = ASTNG_MANAGER.astng_from_module_name(sys.argv[1])
    print repr(obj_code)
    print obj_code.locals().keys()
    #print [str(c.__class__) for c in obj_code.children]
    print obj_code.docstring()
    for c in obj_code.children:
        print c.line, c.num, c


